@@ -6,7 +6,7 @@ isort:skip_file
 from brownie.project import compile_source, run
 from brownie._config import CONFIG as _CONFIG
 from brownie.convert import Fixed, Wei
-from brownie.network import accounts, alert, history, rpc, web3
+from brownie.network import accounts, alert, chain, history, rpc, web3
 from brownie.network.contract import Contract  # NOQA: F401
 
 
@@ -16,6 +16,7 @@ __all__ = [
     "Contract",
     "accounts",  # accounts is an Accounts singleton
     "alert",
+    "chain",
     "history",  # history is a TxHistory singleton
     "network",
     "rpc",  # rpc is a Rpc singleton
