@@ -9,11 +9,13 @@ from typing import Any, Dict, Iterator, List, Optional, Tuple, Union
 
 import eth_account
 import eth_keys
+import rlp
+from eth_utils import keccak
 from hexbytes import HexBytes
 
 from brownie._config import CONFIG, _get_data_folder
 from brownie._singleton import _Singleton
-from brownie.convert import Wei, to_address
+from brownie.convert import EthAddress, Wei, to_address
 from brownie.exceptions import (
     ContractNotFound,
     IncompatibleEVMVersion,
@@ -226,6 +228,11 @@ class Accounts(metaclass=_Singleton):
 
         if acct is None and (address in web3.eth.accounts or force):
             acct = Account(address)
+
+            if CONFIG.network_type == "development" and address not in web3.eth.accounts:
+                # prior to ganache v6.11.0 this does nothing, but should not raise
+                web3.provider.make_request("evm_unlockUnknownAccount", [address])  # type: ignore
+
             self._accounts.append(acct)
 
         if acct:
@@ -295,6 +302,25 @@ class PublicKeyAccount:
     def nonce(self) -> int:
         return web3.eth.getTransactionCount(self.address)
 
+    def get_deployment_address(self, nonce: Optional[int] = None) -> EthAddress:
+        """
+        Return the address of a contract deployed from this account at the given nonce.
+
+        Arguments
+        ---------
+        nonce : int, optional
+            The nonce of the deployment transaction. If not given, the nonce of the next
+            transaction is used.
+        """
+        if nonce is None:
+            nonce = self.nonce
+
+        address = HexBytes(self.address)
+        raw = rlp.encode([address, nonce])
+        deployment_address = keccak(raw)[12:]
+
+        return EthAddress(deployment_address)
+
 
 class _PrivateKeyAccount(PublicKeyAccount):
 
@@ -321,14 +347,25 @@ class _PrivateKeyAccount(PublicKeyAccount):
 
         return nonce
 
-    def _gas_limit(self, to: Optional["Accounts"], amount: int, data: Optional[str] = None) -> int:
+    def _gas_limit(
+        self,
+        to: Optional["Account"],
+        amount: int,
+        gas_price: int,
+        gas_buffer: Optional[float],
+        data: Optional[str] = None,
+    ) -> int:
         gas_limit = CONFIG.active_network["settings"]["gas_limit"]
         if gas_limit == "max":
-            gas_limit = web3.eth.getBlock("latest")["gasLimit"]
-            CONFIG.active_network["settings"]["gas_limit"] = gas_limit
+            return Chain().block_gas_limit
 
         if isinstance(gas_limit, bool) or gas_limit in (None, "auto"):
-            return self.estimate_gas(to, amount, data or "")
+            gas_buffer = gas_buffer or CONFIG.active_network["settings"]["gas_buffer"]
+            gas_limit = self.estimate_gas(to, amount, gas_price, data or "")
+            if gas_buffer != 1:
+                gas_limit = Wei(gas_limit * gas_buffer)
+                return min(gas_limit, Chain().block_gas_limit)
+
         return Wei(gas_limit)
 
     def _gas_price(self) -> Wei:
@@ -350,6 +387,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
         *args: Tuple,
         amount: int = 0,
         gas_limit: Optional[int] = None,
+        gas_buffer: Optional[float] = None,
         gas_price: Optional[int] = None,
         nonce: Optional[int] = None,
         required_confs: int = 1,
@@ -365,6 +403,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
         Kwargs:
             amount: Amount of ether to send with transaction, in wei.
             gas_limit: Gas limit of the transaction.
+            gas_buffer: Multiplier to apply to gas limit.
             gas_price: Gas price of the transaction.
             nonce: Nonce to use for the transaction.
 
@@ -372,6 +411,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
             * Contract instance if the transaction confirms and the contract exists
             * TransactionReceipt if the transaction is pending or reverts
         """
+        if gas_limit and gas_buffer:
+            raise ValueError("Cannot set gas_limit and gas_buffer together")
 
         evm = contract._build["compiler"]["evm_version"]
         if rpc.is_active() and not rpc.evm_compatible(evm):
@@ -383,13 +424,17 @@ class _PrivateKeyAccount(PublicKeyAccount):
             silent = bool(CONFIG.mode == "test" or CONFIG.argv["silent"])
         with self._lock:
             try:
+                gas_price = Wei(gas_price) if gas_price is not None else self._gas_price()
+                gas_limit = Wei(gas_limit) or self._gas_limit(
+                    None, amount, gas_price, gas_buffer, data
+                )
                 txid = self._transact(  # type: ignore
                     {
                         "from": self.address,
                         "value": Wei(amount),
                         "nonce": nonce if nonce is not None else self._pending_nonce(),
-                        "gasPrice": Wei(gas_price) or self._gas_price(),
-                        "gas": Wei(gas_limit) or self._gas_limit(None, amount, data),
+                        "gasPrice": gas_price,
+                        "gas": gas_limit,
                         "data": HexBytes(data),
                     }
                 )
@@ -420,7 +465,12 @@ class _PrivateKeyAccount(PublicKeyAccount):
                     receipt,
                     self.deploy,
                     (contract, *args),
-                    {"amount": amount, "gas_limit": gas_limit, "gas_price": gas_price},
+                    {
+                        "amount": amount,
+                        "gas_limit": gas_limit,
+                        "gas_buffer": gas_buffer,
+                        "gas_price": gas_price,
+                    },
                 ),
                 daemon=True,
             )
@@ -437,26 +487,39 @@ class _PrivateKeyAccount(PublicKeyAccount):
             # if the contract self-destructed during deployment
             return receipt
 
-    def estimate_gas(self, to: "Accounts" = None, amount: int = 0, data: str = None) -> int:
-        """Estimates the gas cost for a transaction. Raises VirtualMachineError
-        if the transaction would revert.
+    def estimate_gas(
+        self, to: "Account" = None, amount: int = 0, gas_price: int = None, data: str = None
+    ) -> int:
+        """
+        Estimate the gas cost for a transaction.
 
-        Args:
-            to: Account instance or address string of transaction recipient.
-            amount: Amount of ether to send in wei.
-            data: Transaction data hexstring.
+        Raises VirtualMachineError if the transaction would revert.
 
-        Returns:
-            Estimated gas value in wei."""
+        Arguments
+        ---------
+        to : Account, optional
+            Account instance or address string of transaction recipient.
+        amount : int, optional
+            Amount of ether to send in wei.
+        gas_price : int, optional
+            Gas price of the transaction.
+        data : str, optional
+            Transaction data hexstring.
+
+        Returns
+        -------
+        Estimated gas value in wei.
+        """
+        tx: Dict = {
+            "from": self.address,
+            "to": str(to) if to else None,
+            "value": Wei(amount),
+            "data": HexBytes(data or ""),
+        }
+        if gas_price is not None:
+            tx["gasPrce"] = gas_price
         try:
-            return web3.eth.estimateGas(
-                {
-                    "from": self.address,
-                    "to": str(to) if to else None,
-                    "value": Wei(amount),
-                    "data": HexBytes(data or ""),
-                }
-            )
+            return web3.eth.estimateGas(tx)
         except ValueError:
             revert_gas_limit = CONFIG.active_network["settings"]["reverting_tx_gas_limit"]
             if revert_gas_limit == "max":
@@ -468,9 +531,10 @@ class _PrivateKeyAccount(PublicKeyAccount):
 
     def transfer(
         self,
-        to: "Accounts" = None,
+        to: "Account" = None,
         amount: int = 0,
         gas_limit: Optional[int] = None,
+        gas_buffer: Optional[float] = None,
         gas_price: Optional[int] = None,
         data: str = None,
         nonce: Optional[int] = None,
@@ -484,6 +548,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
             to: Account instance or address string to transfer to.
             amount: Amount of ether to send, in wei.
             gas_limit: Gas limit of the transaction.
+            gas_buffer: Multiplier to apply to gas limit.
             gas_price: Gas price of the transaction.
             nonce: Nonce to use for the transaction.
             data: Hexstring of data to include in transaction.
@@ -492,15 +557,19 @@ class _PrivateKeyAccount(PublicKeyAccount):
         Returns:
             TransactionReceipt object
         """
+        if gas_limit and gas_buffer:
+            raise ValueError("Cannot set gas_limit and gas_buffer together")
         if silent is None:
             silent = bool(CONFIG.mode == "test" or CONFIG.argv["silent"])
         with self._lock:
+            gas_price = Wei(gas_price) if gas_price is not None else self._gas_price()
+            gas_limit = Wei(gas_limit) or self._gas_limit(to, amount, gas_price, gas_buffer, data)
             tx = {
                 "from": self.address,
                 "value": Wei(amount),
                 "nonce": nonce if nonce is not None else self._pending_nonce(),
-                "gasPrice": Wei(gas_price) if gas_price is not None else self._gas_price(),
-                "gas": Wei(gas_limit) or self._gas_limit(to, amount, data),
+                "gasPrice": gas_price,
+                "gas": gas_limit,
                 "data": HexBytes(data or ""),
             }
             if to:
@@ -522,7 +591,12 @@ class _PrivateKeyAccount(PublicKeyAccount):
         if rpc.is_active():
             undo_thread = threading.Thread(
                 target=Chain()._add_to_undo_buffer,
-                args=(receipt, self.transfer, (to, amount, gas_limit, gas_price, data, None), {}),
+                args=(
+                    receipt,
+                    self.transfer,
+                    (to, amount, gas_limit, gas_buffer, gas_price, data, None),
+                    {},
+                ),
                 daemon=True,
             )
             undo_thread.start()
