@@ -18,7 +18,7 @@ from semantic_version import Version
 from vvm import get_installable_vyper_versions
 from vvm.utils.convert import to_vyper_version
 
-from brownie._config import CONFIG, REQUEST_HEADERS
+from brownie._config import BROWNIE_FOLDER, CONFIG, REQUEST_HEADERS
 from brownie.convert.datatypes import Wei
 from brownie.convert.normalize import format_input, format_output
 from brownie.convert.utils import (
@@ -251,8 +251,9 @@ class ContractConstructor:
             required_confs=tx["required_confs"],
         )
 
-    def _autosuggest(self) -> List:
-        return _contract_method_autosuggest(self)
+    @staticmethod
+    def _autosuggest(obj: "ContractConstructor") -> List:
+        return _contract_method_autosuggest(obj.abi["inputs"], True, obj.payable)
 
     def encode_input(self, *args: tuple) -> str:
         bytecode = self._parent.bytecode
@@ -270,6 +271,35 @@ class ContractConstructor:
         types_list = get_type_strings(self.abi["inputs"])
         return bytecode + eth_abi.encode_abi(types_list, data).hex()
 
+    def estimate_gas(self, *args: Tuple) -> int:
+        """
+        Estimate the gas cost for the deployment.
+
+        Raises VirtualMachineError if the transaction would revert.
+
+        Arguments
+        ---------
+        *args
+            Constructor arguments. The last argument MUST be a dictionary
+            of transaction values containing at minimum a 'from' key to
+            specify which account to deploy this contract from.
+
+        Returns
+        -------
+        int
+            Estimated gas value in wei.
+        """
+        args, tx = _get_tx(None, args)
+        if not tx["from"]:
+            raise AttributeError(
+                "Contract has no owner, you must supply a tx dict"
+                " as the last argument with a 'from' field."
+            )
+
+        return tx["from"].estimate_gas(
+            amount=tx["value"], gas_price=tx["gasPrice"], data=self.encode_input(*args)
+        )
+
 
 class InterfaceContainer:
     """
@@ -279,6 +309,13 @@ class InterfaceContainer:
     def __init__(self, project: Any) -> None:
         self._project = project
 
+        # automatically populate with interfaces in `data/interfaces`
+        # overwritten if a project contains an interface with the same name
+        for path in BROWNIE_FOLDER.glob("data/interfaces/*.json"):
+            with path.open() as fp:
+                abi = json.load(fp)
+            self._add(path.stem, abi)
+
     def _add(self, name: str, abi: List) -> None:
         constructor = InterfaceConstructor(name, abi)
         setattr(self, name, constructor)
@@ -292,6 +329,9 @@ class InterfaceConstructor:
     def __init__(self, name: str, abi: List) -> None:
         self._name = name
         self.abi = abi
+        self.selectors = {
+            build_function_selector(i): i["name"] for i in self.abi if i["type"] == "function"
+        }
 
     def __call__(self, address: str, owner: Optional[AccountsType] = None) -> "Contract":
         return Contract.from_abi(self._name, address, self.abi, owner)
@@ -380,7 +420,10 @@ class _DeployedContractBase(_ContractBase):
     def __getattribute__(self, name: str) -> Any:
         if super().__getattribute__("_reverted"):
             raise ContractNotFound("This contract no longer exists.")
-        return super().__getattribute__(name)
+        try:
+            return super().__getattribute__(name)
+        except AttributeError:
+            raise AttributeError(f"Contract '{self._name}' object has no attribute '{name}'")
 
     def get_method_object(self, calldata: str) -> Optional["_ContractMethod"]:
         """
@@ -658,7 +701,13 @@ class Contract(_DeployedContractBase):
             )
 
         if as_proxy_for is None and data["result"][0].get("Implementation"):
-            as_proxy_for = _resolve_address(data["result"][0]["Implementation"])
+            try:
+                # many proxy patterns use an `implementation()` function, so first we
+                # try to determine the implementation address without trusting etherscan
+                contract = cls.from_abi(name, address, abi)
+                as_proxy_for = contract.implementation()
+            except Exception:
+                as_proxy_for = _resolve_address(data["result"][0]["Implementation"])
 
         if as_proxy_for == address:
             as_proxy_for = None
@@ -682,7 +731,8 @@ class Contract(_DeployedContractBase):
             try:
                 version = Version(compiler_str.lstrip("v")).truncate()
                 is_compilable = (
-                    version
+                    version >= Version("0.4.22")
+                    and version
                     in solcx.get_installable_solc_versions() + solcx.get_installed_solc_versions()
                 )
             except Exception:
@@ -706,7 +756,7 @@ class Contract(_DeployedContractBase):
             evm_version = None
 
         source_str = "\n".join(data["result"][0]["SourceCode"].splitlines())
-        if source_str.startswith("{"):
+        if source_str.startswith("{{"):
             # source was verified using compiler standard JSON
             input_json = json.loads(source_str[1:-1])
             sources = {k: v["content"] for k, v in input_json["sources"].items()}
@@ -719,13 +769,17 @@ class Contract(_DeployedContractBase):
             output_json = compiler.compile_from_input_json(input_json)
             build_json = compiler.generate_build_json(input_json, output_json)
         else:
-            # source was submitted as a single file
-            if compiler_str.startswith("vyper"):
-                path_str = f"{name}.vy"
+            if source_str.startswith("{"):
+                # source was submitted as multiple files
+                sources = {k: v["content"] for k, v in json.loads(source_str).items()}
             else:
-                path_str = f"{name}-flattened.sol"
+                # source was submitted as a single file
+                if compiler_str.startswith("vyper"):
+                    path_str = f"{name}.vy"
+                else:
+                    path_str = f"{name}-flattened.sol"
+                sources = {path_str: source_str}
 
-            sources = {path_str: source_str}
             build_json = compiler.compile_and_format(
                 sources,
                 solc_version=str(version),
@@ -939,8 +993,12 @@ class _ContractMethod:
         else:
             return self.abi["stateMutability"] == "payable"
 
-    def _autosuggest(self) -> List:
-        return _contract_method_autosuggest(self)
+    @staticmethod
+    def _autosuggest(obj: "_ContractMethod") -> List:
+        # this is a staticmethod to be compatible with `_call_suggest` and `_transact_suggest`
+        return _contract_method_autosuggest(
+            obj.abi["inputs"], isinstance(obj, ContractTx), obj.payable
+        )
 
     def info(self) -> None:
         """
@@ -1014,6 +1072,7 @@ class _ContractMethod:
             self._address,
             tx["value"],
             gas_limit=tx["gas"],
+            gas_buffer=tx["gas_buffer"],
             gas_price=tx["gasPrice"],
             nonce=tx["nonce"],
             required_confs=tx["required_confs"],
@@ -1075,6 +1134,36 @@ class _ContractMethod:
             result = result[0]
         return result
 
+    def estimate_gas(self, *args: Tuple) -> int:
+        """
+        Estimate the gas cost for a transaction.
+
+        Raises VirtualMachineError if the transaction would revert.
+
+        Arguments
+        ---------
+        *args
+            Contract method inputs
+
+        Returns
+        -------
+        int
+            Estimated gas value in wei.
+        """
+        args, tx = _get_tx(self._owner, args)
+        if not tx["from"]:
+            raise AttributeError(
+                "Contract has no owner, you must supply a tx dict"
+                " as the last argument with a 'from' field."
+            )
+
+        return tx["from"].estimate_gas(
+            to=self._address,
+            amount=tx["value"],
+            gas_price=tx["gasPrice"],
+            data=self.encode_input(*args),
+        )
+
 
 class ContractTx(_ContractMethod):
     """
@@ -1177,6 +1266,7 @@ def _get_tx(owner: Optional[AccountsType], args: Tuple) -> Tuple:
         "from": owner,
         "value": 0,
         "gas": None,
+        "gas_buffer": None,
         "gasPrice": None,
         "nonce": None,
         "required_confs": 1,
@@ -1187,6 +1277,10 @@ def _get_tx(owner: Optional[AccountsType], args: Tuple) -> Tuple:
         for key, target in [("amount", "value"), ("gas_limit", "gas"), ("gas_price", "gasPrice")]:
             if key in tx:
                 tx[target] = tx[key]
+
+    if isinstance(tx["from"], str):
+        tx["from"] = accounts.at(tx["from"], force=True)
+
     return args, tx
 
 
@@ -1260,20 +1354,6 @@ def _print_natspec(natspec: Dict) -> None:
     print()
 
 
-def _contract_method_autosuggest(method: Any) -> List:
-    types_list = get_type_strings(method.abi["inputs"], {"fixed168x10": "decimal"})
-    params = zip([i["name"] for i in method.abi["inputs"]], types_list)
-
-    if isinstance(method, ContractCall):
-        tx_hint: List = []
-    elif method.payable:
-        tx_hint = [" {'from': Account", " 'value': Wei}"]
-    else:
-        tx_hint = [" {'from': Account}"]
-
-    return [f" {i[1]}{' '+i[0] if i[0] else ''}" for i in params] + tx_hint
-
-
 def _fetch_from_explorer(address: str, action: str, silent: bool) -> Dict:
     url = CONFIG.active_network.get("explorer")
     if url is None:
@@ -1308,3 +1388,42 @@ def _fetch_from_explorer(address: str, action: str, silent: bool) -> Dict:
         raise ValueError(f"Failed to retrieve data from API: {data['result']}")
 
     return data
+
+
+# console auto-completion logic
+
+
+def _call_autosuggest(method: Any) -> List:
+    # since methods are not unique for each object, we use `__reduce__`
+    # to locate the specific object so we can access the correct ABI
+    method = method.__reduce__()[1][0]
+    return _contract_method_autosuggest(method.abi["inputs"], False, False)
+
+
+def _transact_autosuggest(method: Any) -> List:
+    method = method.__reduce__()[1][0]
+    return _contract_method_autosuggest(method.abi["inputs"], True, method.payable)
+
+
+# assign the autosuggest functionality to various methods
+ContractConstructor.encode_input.__dict__["_autosuggest"] = _call_autosuggest
+_ContractMethod.call.__dict__["_autosuggest"] = _call_autosuggest
+_ContractMethod.encode_input.__dict__["_autosuggest"] = _call_autosuggest
+
+ContractConstructor.estimate_gas.__dict__["_autosuggest"] = _transact_autosuggest
+_ContractMethod.estimate_gas.__dict__["_autosuggest"] = _transact_autosuggest
+_ContractMethod.transact.__dict__["_autosuggest"] = _transact_autosuggest
+
+
+def _contract_method_autosuggest(args: List, is_transaction: bool, is_payable: bool) -> List:
+    types_list = get_type_strings(args, {"fixed168x10": "decimal"})
+    params = zip([i["name"] for i in args], types_list)
+
+    if not is_transaction:
+        tx_hint: List = []
+    elif is_payable:
+        tx_hint = [" {'from': Account", " 'value': Wei}"]
+    else:
+        tx_hint = [" {'from': Account}"]
+
+    return [f" {i[1]}{' '+i[0] if i[0] else ''}" for i in params] + tx_hint
