@@ -506,7 +506,7 @@ class TransactionReceipt:
                         step = trace[i]
                 self._revert_msg = pc_map[step["pc"]]["dev"]
                 return
-            except (KeyError, AttributeError):
+            except (KeyError, AttributeError, TypeError):
                 pass
 
         step = next(i for i in trace[::-1] if i["op"] in ("REVERT", "INVALID"))
@@ -525,10 +525,12 @@ class TransactionReceipt:
             offset: Start and end offset associated source code
         }
         """
-        if self._trace is not None:
-            return
         if self._raw_trace is None:
             self._get_trace()
+        if self._trace is not None:
+            # in case `_get_trace` also expanded the trace, do not repeat
+            return
+
         self._trace = trace = self._raw_trace
         self._new_contracts = []
         self._internal_transfers = []
@@ -536,8 +538,6 @@ class TransactionReceipt:
         if self.contract_address or not trace:
             coverage._add_transaction(self.coverage_hash, {})
             return
-        if "fn" in trace[0]:
-            return
 
         if trace[0]["depth"] == 1:
             self._trace_origin = "geth"
@@ -598,7 +598,7 @@ class TransactionReceipt:
                         function=fn._input_sig,
                     )
                 elif calldata:
-                    self._subcalls[-1]["calldata"] = calldata
+                    self._subcalls[-1]["calldata"] = calldata.hex()
 
             # update trace from last_map
             last = last_map[trace[i]["depth"]]
@@ -616,9 +616,11 @@ class TransactionReceipt:
                     last["address"], trace[i]["stack"][-2][-40:], trace[i]["stack"][-3]
                 )
 
-            if not last["pc_map"]:
+            try:
+                pc = last["pc_map"][trace[i]["pc"]]
+            except (KeyError, TypeError):
+                # we don't have enough information about this contract
                 continue
-            pc = last["pc_map"][trace[i]["pc"]]
 
             if trace[i]["depth"] and opcode in ("RETURN", "REVERT", "INVALID", "SELFDESTRUCT"):
                 subcall: dict = next(
@@ -626,14 +628,18 @@ class TransactionReceipt:
                 )
 
                 if opcode == "RETURN":
-                    data = _get_memory(trace[i], -1)
-                    subcall["return_value"] = None
-                    if data:
+                    returndata = _get_memory(trace[i], -1)
+                    if returndata:
                         fn = last["function"]
-                        return_values = fn.decode_output(data)
-                        if len(fn.abi["outputs"]) == 1:
-                            return_values = (return_values,)
-                        subcall["return_value"] = return_values
+                        try:
+                            return_values = fn.decode_output(returndata)
+                            if len(fn.abi["outputs"]) == 1:
+                                return_values = (return_values,)
+                            subcall["return_value"] = return_values
+                        except Exception:
+                            subcall["returndata"] = returndata.hex()
+                    else:
+                        subcall["return_value"] = None
                 elif opcode == "SELFDESTRUCT":
                     subcall["selfdestruct"] = True
                 else:
@@ -1010,6 +1016,13 @@ def _step_internal(
     return key
 
 
+def _format(value: Any) -> str:
+    if isinstance(value, (list, tuple)):
+        mode = black.FileMode(line_length=60)
+        black.format_str(str(value), mode=mode)
+    return str(value)
+
+
 def _step_external(
     step: Dict,
     last_step: Dict,
@@ -1023,7 +1036,6 @@ def _step_external(
     if not expand:
         return key
 
-    mode = black.FileMode(line_length=60)
     result: OrderedDict = OrderedDict({key: {}})
     result[key][f"address: {step['address']}"] = None
 
@@ -1031,10 +1043,10 @@ def _step_external(
         result[key][f"value: {subcall['value']}"] = None
 
     if "inputs" not in subcall:
-        result[key][f"calldata: {subcall['calldata']}"] = None
-    if subcall["inputs"]:
+        result[key][f"calldata: {subcall.get('calldata')}"] = None
+    elif subcall["inputs"]:
         result[key]["input arguments:"] = [
-            f"{k}: {black.format_str(str(v), mode=mode)}" for k, v in subcall["inputs"].items()
+            f"{k}: {_format(v)}" for k, v in subcall["inputs"].items()
         ]
     else:
         result[key]["input arguments: None"] = None
@@ -1042,12 +1054,13 @@ def _step_external(
     if "return_value" in subcall:
         value = subcall["return_value"]
         if isinstance(value, tuple) and len(value) > 1:
-            result[key]["return values:"] = [black.format_str(str(i), mode=mode) for i in value]
+            result[key]["return values:"] = [_format(i) for i in value]
         else:
             if isinstance(value, tuple):
                 value = value[0]
-            value_str = black.format_str(str(value), mode=mode)
-            result[key][f"return value: {value_str}"] = None
+            result[key][f"return value: {_format(value)}"] = None
+    elif "returndata" in subcall:
+        result[key][f"returndata: {subcall['returndata']}"] = None
 
     if "revert_msg" in subcall:
         result[key][f"revert reason: {color('bright red')}{subcall['revert_msg']}{color}"] = None
