@@ -4,18 +4,20 @@ import json
 import os
 import time
 from pathlib import Path
-from typing import Dict, Optional
+from typing import Callable, Dict, List, Optional, Set
 
 from ens import ENS
 from web3 import HTTPProvider, IPCProvider
 from web3 import Web3 as _Web3
 from web3 import WebsocketProvider
+from web3.exceptions import ExtraDataLengthError
+from web3.gas_strategies.rpc import rpc_gas_price_strategy
+from web3.middleware import geth_poa_middleware
 
 from brownie._config import CONFIG, _get_data_folder
 from brownie.convert import to_address
 from brownie.exceptions import MainnetUndefined, UnsetENSName
 
-__tracebackhide__ = True
 _chain_uri_cache: Dict = {}
 
 
@@ -30,25 +32,56 @@ class Web3(_Web3):
         self._mainnet_w3: Optional[_Web3] = None
         self._genesis_hash: Optional[str] = None
         self._chain_uri: Optional[str] = None
+        self._custom_middleware: Set = set()
 
-    def connect(self, uri: str) -> None:
+    def connect(self, uri: str, timeout: int = 30) -> None:
         """Connects to a provider"""
+        for middleware in self._custom_middleware:
+            self.middleware_onion.remove(middleware)
+        self._custom_middleware.clear()
+        self.provider = None
+
         uri = _expand_environment_vars(uri)
         try:
             if Path(uri).exists():
-                self.provider = IPCProvider(uri)
-                return
+                self.provider = IPCProvider(uri, timeout=timeout)
         except OSError:
             pass
-        if uri[:3] == "ws:":
-            self.provider = WebsocketProvider(uri)
-        elif uri[:4] == "http":
-            self.provider = HTTPProvider(uri, {"timeout": 30})
-        else:
-            raise ValueError(
-                "Unknown URI - must be a path to an IPC socket, a websocket "
-                "beginning with 'ws' or a URL beginning with 'http'"
-            )
+
+        if self.provider is None:
+            if uri.startswith("ws"):
+                self.provider = WebsocketProvider(uri, {"close_timeout": timeout})
+            elif uri.startswith("http"):
+
+                self.provider = HTTPProvider(uri, {"timeout": timeout})
+            else:
+                raise ValueError(
+                    "Unknown URI - must be a path to an IPC socket, a websocket "
+                    "beginning with 'ws' or a URL beginning with 'http'"
+                )
+
+        try:
+            if not self.isConnected():
+                return
+        except Exception:
+            # checking an invalid connection sometimes raises on windows systems
+            return
+
+        # add middlewares
+        try:
+            if "fork" in CONFIG.active_network["cmd_settings"]:
+                self._custom_middleware.add(_ForkMiddleware)
+                self.middleware_onion.add(_ForkMiddleware)
+        except (ConnectionError, KeyError):
+            pass
+
+        try:
+            self.eth.getBlock("latest")
+        except ExtraDataLengthError:
+            self._custom_middleware.add(geth_poa_middleware)
+            self.middleware_onion.inject(geth_poa_middleware, layer=0)
+        except ConnectionError:
+            pass
 
     def disconnect(self) -> None:
         """Disconnects from a provider"""
@@ -98,6 +131,32 @@ class Web3(_Web3):
         return _chain_uri_cache[self.genesis_hash]
 
 
+class _ForkMiddleware:
+
+    """
+    Web3 middleware for raising more expressive exceptions when a forked local network
+    cannot access archival states.
+    """
+
+    def __init__(self, make_request: Callable, w3: _Web3):
+        self.w3 = w3
+        self.make_request = make_request
+
+    def __call__(self, method: str, params: List) -> Dict:
+        response = self.make_request(method, params)
+        err_msg = response.get("error", {}).get("message", "")
+        if (
+            err_msg == "Returned error: project ID does not have access to archive state"
+            or err_msg.startswith("Returned error: missing trie node")
+        ):
+            raise ValueError(
+                "Local fork was created more than 128 blocks ago and you do not"
+                " have access to archival states. Please restart your session."
+            )
+
+        return response
+
+
 def _expand_environment_vars(uri: str) -> str:
     if "$" not in uri:
         return uri
@@ -131,6 +190,7 @@ def _resolve_address(domain: str) -> str:
 
 
 web3 = Web3()
+web3.eth.setGasPriceStrategy(rpc_gas_price_strategy)
 
 try:
     with _get_path().open() as fp:
